#include "../include/aprecv-general.h"

int print_802_11_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  struct wvlan_pkt_header *wvlan_header;
  unsigned short type, subtype, to_ds, from_ds, fc, seq_control;
  short i;

  wvlan_header 	= (struct wvlan_pkt_header *)(packet+opt_size);
  fc 		= wvlan_header->fc;
  type 		= ((fc & 0xc) >> 2);
  subtype 	= ((fc & 0xf0) >> 4);
  to_ds 	= (fc & 0x0100);
  from_ds 	= (fc & 0x0200);
  
  apsr_output("\n- 802.11 Header -\n");

  if(header->caplen < APSR_IEEE802_11_FC_LEN) {
	  apsr_output("Error: Captured data length(%d bytes) < Frame Control length.\n", (int) header->caplen);
	  if(statistic)
		  stats.tr.ieee80211++;
	  goto end_ieee80211;
  }

  apsr_output("  Version: %d, Type: %d", (fc & 0x3), type);
  if(verbose) {
	  switch(type) {
		  case APSR_IEEE802_11_TYPE_MGMT:
		    apsr_output(" (Management)");
		    break;
		  case APSR_IEEE802_11_TYPE_CTRL:
		    apsr_output(" (Control)");
		    break;
		  case APSR_IEEE802_11_TYPE_DATA:
		    apsr_output(" (Data)");
		    break;
		  case APSR_IEEE802_11_TYPE_RESERVED:
		    apsr_output(" (Reserved)");
		    break;
		  default:
		    break;
	   }
  }
  apsr_output("\n  Subtype: %d", subtype);
  if(verbose) {
	  switch(type) {
	    case APSR_IEEE802_11_TYPE_MGMT:
	      switch(subtype) {
		      case APSR_IEEE802_11_SUBTYPE_MGMT_ASSOC_REQ:
			      apsr_output(" (Association request)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_MGMT_ASSOC_RES:
			      apsr_output(" (Association response)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_MGMT_REASS_REQ:
			      apsr_output(" (Reassociation request)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_MGMT_REASS_RES:
			      apsr_output(" (Reassociation response)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_MGMT_PROBE_REQ:
		      	      apsr_output(" (Probe request)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_MGMT_PROBE_RES:
			      apsr_output(" (Probe response)");
			      break;
	              case APSR_IEEE802_11_SUBTYPE_MGMT_BEACON:
		              apsr_output(" (Beacon)");
		              break;
		      case APSR_IEEE802_11_SUBTYPE_MGMT_ATIM:
		              apsr_output(" (ATIM)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_MGMT_DISASSOCI:
		              apsr_output(" (Disassociation)");
		              break;
	              case APSR_IEEE802_11_SUBTYPE_MGMT_AUTHENTIC:
		              apsr_output(" (Authentication)");
		              break;
	              case APSR_IEEE802_11_SUBTYPE_MGMT_DEAUTHENT:
		              apsr_output(" (Deauthentication)");
		              break;
		      default:
		              break;	      
              }		      
	      break;
	    case APSR_IEEE802_11_TYPE_CTRL:
	      switch(subtype) {
		      case APSR_IEEE802_11_SUBTYPE_CTRL_PS_POLL:
			      apsr_output(" (Power save poll)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_CTRL_RTS:
			      apsr_output(" (Request to send)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_CTRL_CTS:
			      apsr_output(" (Clear to send)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_CTRL_ACK:
		   	      apsr_output(" (Acknowledgement)");
		              break;
	              case APSR_IEEE802_11_SUBTYPE_CTRL_CF:
			      apsr_output(" (Contention free)");
		              break;
		      case APSR_IEEE802_11_SUBTYPE_CTRL_END_ACK:
		              apsr_output(" (Acknowledgement end)");
		              break;
		      default:
		              break;
	      }	      
	      break;
	    case APSR_IEEE802_11_TYPE_DATA:
	      switch(subtype) {
		      case APSR_IEEE802_11_SUBTYPE_DATA_DATA:
			      apsr_output(" (Data)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_DATA_CF_ACK:
			      apsr_output(" (Contention free ack)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_DATA_CF_POLL:
	       		      apsr_output(" (Contention free poll)");
	                      break;
	              case APSR_IEEE802_11_SUBTYPE_DATA_CF_A_POLL:
		              apsr_output(" (Contention free poll and ack)");
	                      break;
	              case APSR_IEEE802_11_SUBTYPE_DATA_NULL:
	                      apsr_output(" (Null)");
	                      break;
		      case APSR_IEEE802_11_SUBTYPE_DATA_N_CF_ACK:
			      apsr_output(" (Contention free ack without data)");
			      break;
	              case APSR_IEEE802_11_SUBTYPE_DATA_N_CF_POLL:
			      apsr_output(" (Contention free poll without data)");
			      break;
		      case APSR_IEEE802_11_SUBTYPE_DATA_N_CF_A_P:
			      apsr_output(" (Contention free poll and ack without data)");
			      break;
		      default:
			      break;
	      }  
	      break;
	    default:
	      break;
	  }
  }
  apsr_output("\n");		      
  switch(type) {
	   case APSR_IEEE802_11_TYPE_MGMT:
	     apsr_output("  Duration/ID: %d\n", wvlan_header->ext.mgmt.duration);
	     apsr_output("  DA: %s", ether2str(wvlan_header->ext.mgmt.da));
	     apsr_output(", SA: %s\n", ether2str(wvlan_header->ext.mgmt.sa));
	     apsr_output("  BSSID: %s\n", ether2str(wvlan_header->ext.mgmt.bssid));
             /* TODO host order check */
             seq_control = wvlan_header->ext.mgmt.seq_ctrl;
             apsr_output("  Fragment Number: %d", (seq_control & 0x000f));
             apsr_output(", Sequence Number: %d\n",((seq_control & 0xfff0) >> 4));
             if(subtype == APSR_IEEE802_11_SUBTYPE_MGMT_BEACON) {
               apsr_output("  Timestamp: 0x");
               for(i = 7; i >= 0; i--) {
                 apsr_output("%02x", wvlan_header->ext.mgmt.beacon.fixed.time[i]);
               }
               apsr_output("\n  Interval: %u\n", 
                      wvlan_header->ext.mgmt.beacon.fixed.interval);
               apsr_output("  Capabilities: 0x%04x\n", 
                      wvlan_header->ext.mgmt.beacon.fixed.capa);
             } 
             break;
	   case APSR_IEEE802_11_TYPE_CTRL:
	     switch(subtype) {
		     case APSR_IEEE802_11_SUBTYPE_CTRL_PS_POLL:
			     apsr_output("  AID: %d\n",
			      wvlan_header->ext.ctrl_ps_poll.aid); 
                             apsr_output("  BSSID: %s\n",
			      ether2str(wvlan_header->ext.ctrl_ps_poll.bssid)); 
			     apsr_output("  TA: %s\n",
	                      ether2str(wvlan_header->ext.ctrl_ps_poll.ta));
			     apsr_output("  FCS: 0x%2.2x%2.2x%2.2x%2.2x\n", 
			      wvlan_header->ext.ctrl_ps_poll.fcs[0],
			      wvlan_header->ext.ctrl_ps_poll.fcs[1],
			      wvlan_header->ext.ctrl_ps_poll.fcs[2],
			      wvlan_header->ext.ctrl_ps_poll.fcs[3]);
			     break;
		     case APSR_IEEE802_11_SUBTYPE_CTRL_RTS:
			     apsr_output("  Duration/ID: %d\n", 
			      wvlan_header->ext.ctrl_rts.duration);
                             apsr_output("  RA: %s", 
			      ether2str(wvlan_header->ext.ctrl_rts.ra));
                             apsr_output(", TA: %s\n", 
			      ether2str(wvlan_header->ext.ctrl_rts.ta));
                             apsr_output("\n  FCS: 0x%2.2x%2.2x%2.2x%2.2x\n",
			      wvlan_header->ext.ctrl_rts.fcs[0],
			      wvlan_header->ext.ctrl_rts.fcs[1],
			      wvlan_header->ext.ctrl_rts.fcs[2],
			      wvlan_header->ext.ctrl_rts.fcs[3]);
			     break;
		     case APSR_IEEE802_11_SUBTYPE_CTRL_CTS:
                             apsr_output("  Duration/ID: %d\n",
		              wvlan_header->ext.ctrl_cts.duration);
                             apsr_output("  RA: %s",
			      ether2str(wvlan_header->ext.ctrl_cts.ra));
			     apsr_output("\n  FCS: 0x%2.2x%2.2x%2.2x%2.2x\n",
			      wvlan_header->ext.ctrl_cts.fcs[0],
			      wvlan_header->ext.ctrl_cts.fcs[1],
			      wvlan_header->ext.ctrl_cts.fcs[2],
			      wvlan_header->ext.ctrl_cts.fcs[3]);
			     break;				 
		     case APSR_IEEE802_11_SUBTYPE_CTRL_ACK:
                             apsr_output("  Duration/ID: %d\n",
		              wvlan_header->ext.ctrl_ack.duration);
                             apsr_output("  RA: %s",
			      ether2str(wvlan_header->ext.ctrl_ack.ra)); 
                             apsr_output("\n  FCS: 0x%2.2x%2.2x%2.2x%2.2x\n",
		              wvlan_header->ext.ctrl_ack.fcs[0],
		              wvlan_header->ext.ctrl_ack.fcs[1],
		              wvlan_header->ext.ctrl_ack.fcs[2],
		              wvlan_header->ext.ctrl_ack.fcs[3]);
                             break;
		     case APSR_IEEE802_11_SUBTYPE_CTRL_CF:
                             apsr_output("  Duration/ID: %d\n",
                              wvlan_header->ext.ctrl_end.duration);
                             apsr_output("  RA: %s",
	                      ether2str(wvlan_header->ext.ctrl_end.ra));
                             apsr_output("  BSSID: %s\n",
			      ether2str(wvlan_header->ext.ctrl_end.bssid));
			     apsr_output("\n  FCS: 0x%2.2x%2.2x%2.2x%2.2x\n",
		              wvlan_header->ext.ctrl_end.fcs[0],
		              wvlan_header->ext.ctrl_end.fcs[1],
		              wvlan_header->ext.ctrl_end.fcs[2],
		              wvlan_header->ext.ctrl_end.fcs[3]);
                             break;
	             case APSR_IEEE802_11_SUBTYPE_CTRL_END_ACK:
                             apsr_output("  Duration/ID: %d\n",
                              wvlan_header->ext.ctrl_end_ack.duration);
                             apsr_output("  RA: %s",
	                      ether2str(wvlan_header->ext.ctrl_end_ack.ra));
                             apsr_output("  BSSID: %s\n",
                              ether2str(wvlan_header->ext.ctrl_end_ack.bssid));
                             apsr_output("\n  FCS: 0x%2.2x%2.2x%2.2x%2.2x\n",
                              wvlan_header->ext.ctrl_end_ack.fcs[0],
                              wvlan_header->ext.ctrl_end_ack.fcs[1],
                              wvlan_header->ext.ctrl_end_ack.fcs[2],
                              wvlan_header->ext.ctrl_end_ack.fcs[3]);
                             break;
		     default:
			     break; 
             }
	     break;
	   case APSR_IEEE802_11_TYPE_DATA:
             apsr_output("  Duration/ID: %d\n", wvlan_header->ext.data.duration);
	     apsr_output("  More-Fragment: %d", (fc & 0x0400));
             apsr_output(", Retry: %d\n", (fc & 0x0800));
             apsr_output("  Power-Management: %d", (fc & 0x1000));
             apsr_output(", More-Data: %d\n", (fc & 0x2000));
             apsr_output("  WEP: %d", (fc & 0x4000));
             apsr_output(", Order: %d\n", (fc & 0x8000));
	     if(!to_ds) {
		     if(!from_ds) {	/* to_ds = 0; from_ds = 0 */
		           apsr_output("  DA: %s\n", 
				ether2str(wvlan_header->ext.data.addr1));
                           apsr_output("  SA: %s\n",
	                        ether2str(wvlan_header->ext.data.addr2));   
                           apsr_output("  RA: %s\n",
			        ether2str(wvlan_header->ext.data.addr3)); 
			   opt_size = opt_size + 24 - data_link_size;
	     	     }
	   	     else {	/* to_ds = 0; from_ds = 1 */
                           apsr_output("  DA: %s\n",
                                ether2str(wvlan_header->ext.data.addr1));
                           apsr_output("  BSSID: %s\n",
				ether2str(wvlan_header->ext.data.addr2));	
                           apsr_output("  SA: %s\n",
                                ether2str(wvlan_header->ext.data.addr3));
			   opt_size = opt_size + 24 - data_link_size;
	             }
	     }
	     else {
		    if(!from_ds) { /* to_ds = 1; from_ds = 0 */
                           apsr_output("  BSSID: %s\n",
                                ether2str(wvlan_header->ext.data.addr1));
                           apsr_output("  SA: %s\n",
                                ether2str(wvlan_header->ext.data.addr2));
                           apsr_output("  DA: %s\n",
                                ether2str(wvlan_header->ext.data.addr3));
			   opt_size = opt_size + 24 - data_link_size;
                     }  
		     else {	/* to_ds = 1; from_ds = 1 */		  
                           apsr_output("  RA: %s\n",
                                ether2str(wvlan_header->ext.data.addr1));
                           apsr_output("  TA: %s\n",
                                ether2str(wvlan_header->ext.data.addr2));
                           apsr_output("  DA: %s\n",
                                ether2str(wvlan_header->ext.data.addr3));
                           apsr_output("  SA: %s\n",
                                ether2str(wvlan_header->ext.data.addr4));
			   opt_size = opt_size + 30 - data_link_size;
                     }
	     }
	     /* TODO host order check */
	     seq_control = wvlan_header->ext.data.seq_ctrl;
	     apsr_output("  Fragment Number: %d", (seq_control & 0x000f));
	     apsr_output(", Sequence Number: %d",((seq_control & 0xfff0) >> 4));
	     if((fc & 0x4000)) 
		     apsr_output("\n  Data: <WEP encrypted, not implemented @ the moment>\n");
	     else {
		     print_llc_packet(header, packet, opt_size, 0);
	     }
	     break;
	   default:
	     break;
  }
  
  if(statistic) 
	  stats.c.ieee80211++;
  
end_ieee80211:
  
  return(RET_SUCCESS);  
}
