#include "../include/aprecv-general.h"

int print_gre_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct gre_pkt_header *gre_header;
  static unsigned short flags, protocol;

  gre_header 	= (struct gre_pkt_header *)(packet+data_link_size+opt_size);
  flags 	= apsr_ntohs(gre_header->flags);
  protocol 	= apsr_ntohs(gre_header->protocol);
  
  if((flags & APSR_GRE_FLAGS_VERSION) == 0) {
	  apsr_output("- GREv0 Header -\n  Version: %d\n", 
			  flags & APSR_GRE_FLAGS_VERSION);
	  apsr_output("  Checksum present: %d, ", (flags & APSR_GRE_FLAGS_CHECKSUM)>>15);
          apsr_output("Routing present: %d, ", (flags & APSR_GRE_FLAGS_ROUTING)>>14);
          apsr_output("Key present: %d\n", (flags & APSR_GRE_FLAGS_KEY)>>13);
	  apsr_output("  Sequence present: %d, ", (flags & APSR_GRE_FLAGS_SEQUENCE)>>12); 
          apsr_output("Strict source route: %d, ", (flags & APSR_GRE_FLAGS_STRICTSOURCE)>>11);
          apsr_output("Recursion control: %d\n", (flags & APSR_GRE_FLAGS_RECURSE)>>9);
	  apsr_output("  Protocol: 0x%04x", protocol);
          if(verbose) 
		  print_ethernet_proto(protocol);
          apsr_output("\n");
 
/*
 *		 - TODO - 
 *	   Print optional fields
 *
 */
		  
	  if(statistic)
		  stats.c.grev0++;		  	  
  }
  else if((flags & APSR_GRE_FLAGS_VERSION) == 1) {
          apsr_output("- GREv1 Header -\n  Version: %d\n",
                          flags & APSR_GRE_FLAGS_VERSION);
          apsr_output("  Checksum present: %d, ", (flags & APSR_GRE_FLAGS_CHECKSUM)>>15);
          apsr_output("Routing present: %d, ", (flags & APSR_GRE_FLAGS_ROUTING)>>14);
          apsr_output("Key present: %d\n", (flags & APSR_GRE_FLAGS_KEY)>>13);
          apsr_output("  Sequence present: %d, ", (flags & APSR_GRE_FLAGS_SEQUENCE)>>12);
          apsr_output("Strict source route: %d, ", (flags & APSR_GRE_FLAGS_STRICTSOURCE)>>11);
          apsr_output("Recursion control: %d\n", (flags & APSR_GRE_FLAGS_RECURSE)>>9);
	  apsr_output("  Acknowledgement present: %d\n", (flags & APSR_GRE_FLAGS_ACK)>>8);
          apsr_output("  Protocol: 0x%04x", protocol);
	  if(verbose)
		  print_ethernet_proto(protocol);
	  apsr_output("\n");

/*
 *               - TODO -
 *         Print optional fields
 *
 */

	  if(statistic)
		  stats.c.grev1++;
  }	 
  else {
	apsr_output("- Not supported GRE(%d) version -\n",
			flags & APSR_GRE_FLAGS_VERSION);

	goto gre_end;
  }
	
  /* add 4 bytes for Flags + Checksum */ 
  opt_size = opt_size + 4;

  /* check for optional data */
  if(((flags & APSR_GRE_FLAGS_CHECKSUM)>>15) == 1) {
    /* 2 byte checksum + 2 byte offset */
    opt_size = opt_size + 4;
  }
  if(((flags & APSR_GRE_FLAGS_ROUTING)>>14) == 1) {
    opt_size = opt_size + 4;
  }
  if(((flags & APSR_GRE_FLAGS_KEY)>>13) == 1) {
    opt_size = opt_size + 4;
  }
  if(((flags & APSR_GRE_FLAGS_SEQUENCE)>>12) == 1) {
    opt_size = opt_size + 4;
  }
  switchto_ethernet_proto(header, packet, opt_size, protocol);
  
gre_end:
  
  return(RET_SUCCESS);
}
