#include "../include/aprecv-general.h"

int print_icmpv6_packet (unsigned char *packet, int opt_size) {
  static struct icmpv6_pkt_header *icmpv6_header;

  icmpv6_header = (struct icmpv6_pkt_header *)(packet+data_link_size+opt_size);

  apsr_output("- ICMPv6 Header -\n  Type: %d",(int) icmpv6_header->type);

  if(verbose) {
	  switch(icmpv6_header->type) {
		  case APSR_ICMP6_DEST_UNREA:
		    apsr_output(" (Dest. unreachable)");
		    break;
		  case APSR_ICMP6_PACKET_BIG:
		    apsr_output(" (Packet too big)");
		    break;
		  case APSR_ICMP6_TIME_EXCEE:
		    apsr_output(" (Time exceeded)");
		    break;
		  case APSR_ICMP6_PARA_PROBL:
		    apsr_output(" (Parameter problem)");
		    break;
		  case APSR_ICMP6_ECHO_REQUE:
		    apsr_output(" (Echo request)");
		    break;
		  case APSR_ICMP6_ECHO_REPLY:
		    apsr_output(" (Echo reply)");
		    break;
		  case APSR_ICMP6_GROUP_QUER:
		    apsr_output(" (Group Membership Query)");
		    break;
		  case APSR_ICMP6_GROUP_REPO:
		    apsr_output(" (Group Membership Report)");
	            break;
		  case APSR_ICMP6_GROUP_REDU:
		    apsr_output(" (Group Membership Reduction)");
		    break;
		  case APSR_ICMP6_ROUTER_SOL:
		    apsr_output(" (Router Solicitation)");
		    break;
		  case APSR_ICMP6_ROUTER_ADV:
		    apsr_output(" (Router Advertisement)");
		    break;
		  case APSR_ICMP6_NEIGHB_SOL:
		    apsr_output(" (Neighbor Solicitation)");
		    break;
		  case APSR_ICMP6_NEIGHB_ADV:
		    apsr_output(" (Neighbor Advertisement)");
		    break;
		  case APSR_ICMP6_REDIRECT:
		    apsr_output(" (Redirect)");
		    break;
		  case APSR_ICMP6_ROUTER_REN:
		    apsr_output(" (Router renumbering)");
		    break;
		  case APSR_ICMP6_NODE_QUERY:
		    apsr_output(" (Node Information Query)");
		    break;
		  case APSR_ICMP6_NODE_RESPO:
		    apsr_output(" (Node Information Response)");
		    break;
		  case APSR_ICMP6_INV_ND_SOL:
		    apsr_output(" (Inv. Neighbor Disc. Sol. Message)");
		    break;
		  case APSR_ICMP6_INV_ND_ADV:
                    apsr_output(" (Inv. Neighbor Disc. Adv. Message)");
                    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
	  }
  }

  apsr_output("\n  Code: %d",(int) icmpv6_header->code);

  if(verbose) {
	  if(icmpv6_header->type == APSR_ICMP6_DEST_UNREA) {
		  switch(icmpv6_header->code) {
			  case APSR_ICMP6_CODE_NO_ROUTE:
			    apsr_output(" (No route to destination)\n  ");
			    break;
			  case APSR_ICMP6_CODE_COM_AD_P:
			    apsr_output(" (Com. with dest. admin. prohibited)\n  ");
			    break;
			  case APSR_ICMP6_CODE_NOT_ASSI:
			    apsr_output(" (Not assigned)\n  ");
			    break;
			  case APSR_ICMP6_CODE_ADDR_UNR:
			    apsr_output(" (Address unreachable)\n  ");
	                    break;
                          case APSR_ICMP6_CODE_PORT_UNR:
                            apsr_output(" (Port unreachable)\n  ");
                            break;
                          default:
                            apsr_output(" (unknown)\n  ");			    
		            break;
		  }    	    
	  }
	  else if(icmpv6_header->type == APSR_ICMP6_TIME_EXCEE) {
		  switch(icmpv6_header->code) {
			  case APSR_ICMP6_CODE_HOP_EXCE:
			    apsr_output(" (Hop limit exceeded in transit)\n  ");
			    break;
			  case APSR_ICMP6_CODE_FRA_EXCE:
			    apsr_output(" (Fragment reassembly time exceeded)\n  ");
			    break;
			  default:
			    apsr_output(" (unknown)\n  ");
			    break;
		  }
          }
	  else if(icmpv6_header->type == APSR_ICMP6_PARA_PROBL) {
		  switch(icmpv6_header->code) {
			  case APSR_ICMP6_CODE_ERR_HEAD:
			    apsr_output(" (Errorneous header field encount.)\n  ");
			    break;
			  case APSR_ICMP6_CODE_UNR_NEXT:
			    apsr_output(" (Unrecognized Next H. t. encount.)\n  ");
			    break;
			  case APSR_ICMP6_CODE_IP6_OPTI:
			    apsr_output(" (Unrecognized IPv6 opt. encount.)\n  ");
			    break;
			  default:
			    apsr_output(" (unknown)\n  ");
			    break;
		  }
	  }
	  else if(icmpv6_header->type == APSR_ICMP6_ROUTER_REN) {
		  switch(icmpv6_header->code) {
			  case APSR_ICMP6_CODE_ROU_COMM:
			    apsr_output(" (Router Renumbering Command)\n  ");
			    break;
			  case APSR_ICMP6_CODE_ROU_RESU:
			    apsr_output(" (Router Renumbering Result)\n  ");
			    break;
			  case APSR_ICMP6_CODE_SEQ_RESE:
			    apsr_output(" (Sequence Number Reset)\n  ");
			    break;
			  default:
			    apsr_output(" (unknown)\n  ");
			    break;
	          }		    
          }					    
 	  else {
		  apsr_output("\n  ");
          }
  }	 
  else {
	  apsr_output(", ");
  }	  
   
  apsr_output("Checksum: 0x%04x\n", apsr_ntohs(icmpv6_header->sum));

  switch(icmpv6_header->type) {
	  case APSR_ICMP6_PACKET_BIG:
	    apsr_output("  MTU: %d\n", apsr_ntohl(icmpv6_header->ext.packet_too_big.mtu));
	    break;
	  case APSR_ICMP6_PARA_PROBL:
	    apsr_output("  Pointer: %d\n", apsr_ntohl(icmpv6_header->ext.parameter_problem.pointer));
	    break;
	  case APSR_ICMP6_ECHO_REQUE:
	  case APSR_ICMP6_ECHO_REPLY:
            apsr_output("  Identifier: %d, Seq.: %d\n", 
			    apsr_ntohs(icmpv6_header->ext.echo.identifier),
			    apsr_ntohs(icmpv6_header->ext.echo.seq_num));
            break;
          case APSR_ICMP6_GROUP_QUER:
          case APSR_ICMP6_GROUP_REPO:
          case APSR_ICMP6_GROUP_REDU:
	    apsr_output("  Maximum Response Delay: %d\n", 
			    apsr_ntohs(icmpv6_header->ext.group.mrd));
	    break;
 	  default:
	    break;
  }
  
  if(statistic) 
	  stats.c.icmpv6++;
  
  return(RET_SUCCESS);
}
