#include "../include/aprecv-general.h"

int print_igmp_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct igmpv2_pkt_header *igmpv2_header;
  static char *group_address;
  static unsigned short pkt_checksum, correct_checksum;

  igmpv2_header = (struct igmpv2_pkt_header *)(packet+data_link_size+opt_size);
  group_address = strdup(inet_ntoa(igmpv2_header->group_addr));

  switch(igmpv2_header->type) {
    case APSR_IGMP_CREATE_GROUP_REQ:
    case APSR_IGMP_CREATE_GROUP_REP:
    case APSR_IGMP_JOIN_GROUP_REQ: 
    case APSR_IGMP_JOIN_GROUP_REP:
    case APSR_IGMP_LEAVE_GROUP_REQ:
    case APSR_IGMP_LEAVE_GROUP_REP:
    case APSR_IGMP_CONF_GROUP_REQ:
    case APSR_IGMP_CONF_GROUP_REP:
      apsr_output("- IGMPv0 Header -\n  Type: 0x%02x",(int)igmpv2_header->type);
      if(verbose)          
	      print_igmp_type(igmpv2_header->type);

      pkt_checksum = apsr_ntohs(igmpv2_header->sum);
      apsr_output("\n  Code: %u, Checksum: 0x%04x",
		      (int) igmpv2_header->max_res_time, pkt_checksum);
      
      if(calc_checksums) {
	      igmpv2_header->sum=0x0000;
	      correct_checksum=apsr_ntohs(in_cksum((unsigned short *)igmpv2_header,
				      APSR_IGMPv2_H, 0));
	      if(pkt_checksum == correct_checksum) {
		      apsr_output(" (correct checksum)");
	      }
	      else {
	              apsr_output(" (incorrect checksum - should be 0x%04x)",
				      correct_checksum);
                      if(statistic)
			      stats.cr.igmpv0++;
              }
      }

      apsr_output("\n  Group Addr: %s\n", group_address);

      if(statistic)
	      stats.c.igmpv0++;
      break;
    case APSR_IGMP_MEMBERSHIP_QUERY:
    case APSR_IGMP_MEMBERSHIPV1_REP:
    case APSR_IGMP_DVMRP: 
      apsr_output("- IGMPv1 Header -\n  Type: 0x%02x",(int)igmpv2_header->type);
      if(verbose)          
	      print_igmp_type(igmpv2_header->type);

      pkt_checksum = apsr_ntohs(igmpv2_header->sum);
      apsr_output("\n  Unused: %u, Checksum: 0x%04x",
		      (int) igmpv2_header->max_res_time, pkt_checksum);
      
      if(calc_checksums) {
	      igmpv2_header->sum=0x0000;
	      correct_checksum=apsr_ntohs(in_cksum((unsigned short *)igmpv2_header,
				      APSR_IGMPv2_H, 0));
	      if(pkt_checksum == correct_checksum) {
		      apsr_output(" (correct checksum)");
	      }
	      else {
	              apsr_output(" (incorrect checksum - should be 0x%04x)",
				      correct_checksum);
                      if(statistic)
			      stats.cr.igmpv1++;
              }
      }

      apsr_output("\n  Group Addr: %s\n", group_address);

      if(statistic)
	      stats.c.igmpv1++;
      break;
    case APSR_IGMP_PIMV1:
    case APSR_IGMP_CISCO_TRACE_MESS:
    case APSR_IGMP_MEMBERSHIPV2_REP:
    case APSR_IGMP_MEMBERSHIP_LEAVE:
    case APSR_IGMP_MULTICAST_TRA_RE:
    case APSR_IGMP_MULTICAST_TRACER:
    case APSR_IGMP_MEMBERSHIPV3_REP:
    case APSR_IGMP_MULTICAST_R_ADVE:
    case APSR_IGMP_MULTICAST_R_SOLI:
    case APSR_IGMP_MULTICAST_R_TERM:
      apsr_output("- IGMPv2 Header -\n  Type: 0x%02x",(int)igmpv2_header->type);
      if(verbose)
	      print_igmp_type(igmpv2_header->type);
      
      pkt_checksum = apsr_ntohs(igmpv2_header->sum);
      apsr_output("\n  Max resonse time: %u, Checksum: 0x%04x",
		      (int) igmpv2_header->max_res_time, pkt_checksum);

      if(calc_checksums) {
	      igmpv2_header->sum=0x0000;
	      correct_checksum=apsr_ntohs(in_cksum((unsigned short *)igmpv2_header, APSR_IGMPv2_H, 0));
              if(pkt_checksum == correct_checksum) {
		      apsr_output(" (correct checksum)");
              }
              else {
		      apsr_output(" (incorrect checksum - should be 0x%04x)",
			      correct_checksum);

		      if(statistic)
			      stats.cr.igmpv2++;
              }
      }
  
      /* TODO if APSR_IGMP_PIMV1 or APSR_IGMP_DVMRP jump directly to print-pim.c and later to print-dvmrp.c */
  
      if(igmpv2_header->type != APSR_IGMP_DVMRP) 
	      apsr_output("\n  Group Addr: %s\n", group_address);
      else 
	      apsr_output("\n");

      if(statistic) 
	      stats.c.igmpv2++;
      break;
    case APSR_IGMP_RGMP_LEAVE_GROUP:
    case APSR_IGMP_RGMP_JOIN_GROUP:
    case APSR_IGMP_RGMP_BYE:
    case APSR_IGMP_RGMP_HELLO:
      apsr_output("- RGMP Header -\n  Type: 0x%02x", (int)igmpv2_header->type);
      if(verbose)
	      print_igmp_type(igmpv2_header->type);

      pkt_checksum = apsr_ntohs(igmpv2_header->sum);
      apsr_output("\n  Reserved: %u, Checksum: 0x%04x",
		      (int) igmpv2_header->max_res_time, pkt_checksum);

      if(calc_checksums) {
	      igmpv2_header->sum=0x0000;
	      correct_checksum=apsr_ntohs(in_cksum((unsigned short *)igmpv2_header, APSR_IGMPv2_H, 0));
	      if(pkt_checksum == correct_checksum) {
		      apsr_output(" (correct checksum)");
	      }
	      else {
	              apsr_output(" (incorrect checksum - should be 0x%04x)",
	                      correct_checksum);
              	      if(statistic)
			      stats.cr.rgmp++;
	      }
      }
	      
      if(statistic)
	      stats.c.rgmp++;
      break;
    default:
      apsr_output("- Unknown IGMP Header -\n");
      break;
  }
      
  free(group_address);
  
  return(RET_SUCCESS);
}

void print_igmp_type(unsigned char type) {
	switch(type) {
		case APSR_IGMP_CREATE_GROUP_REQ:
		  apsr_output(" (Create Group Request)");
		  break;
		case APSR_IGMP_CREATE_GROUP_REP:
		  apsr_output(" (Create Group Reply)");
		  break;
		case APSR_IGMP_JOIN_GROUP_REQ:
		  apsr_output(" (Join Group Request)");
		  break;
		case APSR_IGMP_JOIN_GROUP_REP:
		  apsr_output(" (Join Group Reply)");
		  break;
		case APSR_IGMP_LEAVE_GROUP_REQ:
		  apsr_output(" (Leave Group Request)");
		  break;
		case APSR_IGMP_LEAVE_GROUP_REP:
		  apsr_output(" (Leave Group Reply)");
		  break;
		case APSR_IGMP_CONF_GROUP_REQ:
		  apsr_output(" (Confirm Group Request)");
		  break;
		case APSR_IGMP_CONF_GROUP_REP:
		  apsr_output(" (Confirm Group Reply)");
		  break;
		case APSR_IGMP_MEMBERSHIP_QUERY:
                  apsr_output(" (Membership query)");
                  break;
                case APSR_IGMP_MEMBERSHIPV1_REP:
                  apsr_output(" (Membership version 1 report)");
                  break;
                case APSR_IGMP_DVMRP:
                  apsr_output(" (DVMRP)");
                  break;
                case APSR_IGMP_PIMV1:
                  apsr_output(" (PIMv1)");
                  break;
                case APSR_IGMP_CISCO_TRACE_MESS:
                  apsr_output(" (Cisco trace messages)");
                  break;
                case APSR_IGMP_MEMBERSHIPV2_REP:
                  apsr_output(" (Membership version 2 report)");
                  break;
                case APSR_IGMP_MULTICAST_TRA_RE:
                  apsr_output(" (Multicast traceroute respone)");
                  break;
                case APSR_IGMP_MULTICAST_TRACER:
                  apsr_output(" (Multicast traceroute)");
                  break;
                case APSR_IGMP_MEMBERSHIPV3_REP:
                  apsr_output(" (Membership version 3 report)");
                  break;
                case APSR_IGMP_MEMBERSHIP_LEAVE:
                  apsr_output(" (Leave group)");
                  break;
                case APSR_IGMP_MULTICAST_R_ADVE:
                  apsr_output(" (Multicast Router Advertisement)");
                  break;
                case APSR_IGMP_MULTICAST_R_SOLI:
                  apsr_output(" (Multicast Router Solicitation)");
                  break;
                case APSR_IGMP_MULTICAST_R_TERM:
                  apsr_output(" (Multicast Router Termination)");
                  break;
		case APSR_IGMP_RGMP_LEAVE_GROUP:
		  apsr_output(" (Leave a group)");
		  break;
		case APSR_IGMP_RGMP_JOIN_GROUP:
		  apsr_output(" (Join a group)");
		  break;
		case APSR_IGMP_RGMP_BYE:
		  apsr_output(" (Bye)");
		  break;
		case APSR_IGMP_RGMP_HELLO:
		  apsr_output(" (Hello)");
		  break;
                default:
                  apsr_output(" (unkown)");
                  break;
        }
}
