#include "../include/aprecv-general.h"

int print_ipx_packet (unsigned char *packet, int opt_size) {
  static struct ipx_pkt_header *ipx_header;
  static unsigned short socket_dest, socket_src;

  ipx_header  = (struct ipx_pkt_header *)(packet+data_link_size+opt_size);
  socket_dest = apsr_ntohs(ipx_header->socket_dest);
  socket_src  = apsr_ntohs(ipx_header->socket_src);
  
  apsr_output("\n- IPX Header -\n  Checksum: 0x%04x, Length: %d\n", apsr_ntohs(ipx_header->sum), apsr_ntohs(ipx_header->length));
  apsr_output("  Transport Control: %d, Packet Type: %d", ipx_header->trans_control, ipx_header->packet_type);

  if(verbose) {
	  switch(ipx_header->packet_type) {
		  case APSR_IPX_TYPE_UNKNOWN: 
	            apsr_output(" (unknown)");
		    break;
		  case APSR_IPX_TYPE_RIP:
		    apsr_output(" (RIP)");
		    break;
		  case APSR_IPX_TYPE_ECHO:
		    apsr_output(" (Echo)");
		    break;
		  case APSR_IPX_TYPE_ERROR:
		    apsr_output(" (Error)");
		    break;
		  case APSR_IPX_TYPE_IDP:	/* IPX */
		    apsr_output(" (IDP)");
		    break;
		  case APSR_IPX_TYPE_SPP:
		    apsr_output(" (SPP)");		/* SPX */
		    break;
		  case APSR_IPX_TYPE_NCP:
		    apsr_output(" (NCP)");
		    break;
		  case APSR_IPX_TYPE_NETBIOS:
		    apsr_output(" (NetBIOS)");
		    break;
		  case APSR_IPX_TYPE_NDS:
		    apsr_output(" (NDS NCP)");
		    break;
		  default:
		    break;
	  }
  }
	  
  apsr_output("\n  [Dst] Net: %04x%04x, Node: %s, Socket: %d", 
		 apsr_ntohs(ipx_header->net_dest[0]), apsr_ntohs(ipx_header->net_dest[1]),
		 ether2str(ipx_header->node_dest), socket_dest);
  if(verbose) {
	  switch(socket_dest) {
		  case APSR_IPX_SOCKET_NUM_ECH:
		    apsr_output(" (Echo)");
		    break;
		  case APSR_IPX_SOCKET_NUM_ERR:
		    apsr_output(" (Error)");
		    break;
		  case APSR_IPX_SOCKET_NUM_NVT:
		    apsr_output(" (NVT)");
		    break;
		  case APSR_IPX_SOCKET_NUM_NCP:
		    apsr_output(" (NCP)");
		    break;
		  case APSR_IPX_SOCKET_NUM_SAP:
		    apsr_output(" (SAP)");
		    break;
		  case APSR_IPX_SOCKET_NUM_RIP:
		    apsr_output(" (RIP)");
		    break;
                  case APSR_IPX_SOCKET_NUM_NETBIOS:
                    apsr_output(" (NetBIOS)");
                    break;
                  case APSR_IPX_SOCKET_NUM_NDP:
                    apsr_output(" (NDP)");
                    break;
                  case APSR_IPX_SOCKET_NUM_NWLINK:
                    apsr_output(" (NWLink)");
                    break;
                  case APSR_IPX_SOCKET_NUM_NVT2:
                    apsr_output(" (NVT2)");
                    break;
                  case APSR_IPX_SOCKET_NUM_PRNT:
                    apsr_output(" (Print)");
                    break;
                  case APSR_IPX_SOCKET_NUM_EIGRP:
                    apsr_output(" (EIGRP)");
                    break;
		  default:
		    apsr_output(" (unknown)");
		    break;
   	}
  }

  apsr_output("\n  [Src] Net: %04x%04x, Node: %s, Socket: %d", 
		 apsr_ntohs(ipx_header->net_src[0]), apsr_ntohs(ipx_header->net_src[1]),
		 ether2str(ipx_header->node_src), socket_src);
  if(verbose) {
        switch(socket_src) {
                  case APSR_IPX_SOCKET_NUM_ECH:
                    apsr_output(" (Echo)");
                    break;
                  case APSR_IPX_SOCKET_NUM_ERR:
                    apsr_output(" (Error)");
                    break;
                  case APSR_IPX_SOCKET_NUM_NVT:
                    apsr_output(" (NVT)");
                    break;
                  case APSR_IPX_SOCKET_NUM_NCP:
                    apsr_output(" (NCP)");
                    break;
                  case APSR_IPX_SOCKET_NUM_SAP:
                    apsr_output(" (SAP)");
                    break;
                  case APSR_IPX_SOCKET_NUM_RIP:
                    apsr_output(" (RIP)");
                    break;
		  case APSR_IPX_SOCKET_NUM_NETBIOS:
		    apsr_output(" (NetBIOS)");
		    break;
                  case APSR_IPX_SOCKET_NUM_NDP:
                    apsr_output(" (NDP)");
                    break;
		  case APSR_IPX_SOCKET_NUM_NWLINK:
		    apsr_output(" (NWLink)");
		    break;
                  case APSR_IPX_SOCKET_NUM_NVT2:
                    apsr_output(" (NVT2)");
                    break;
                  case APSR_IPX_SOCKET_NUM_PRNT:
                    apsr_output(" (Print)");
                    break;
		  case APSR_IPX_SOCKET_NUM_EIGRP:
		    apsr_output(" (EIGRP)");
		    break;
                  default:
                    apsr_output(" (unknown)");
                    break;
        }
  }	  
  
  switch(ipx_header->packet_type) {
	  case APSR_IPX_TYPE_RIP:
            print_ipxrip_packet(packet, opt_size+30);
	    break;
	  case APSR_IPX_TYPE_SPP:
	    print_spx_packet(packet, opt_size+30);
	    break;
	  default:
	    apsr_output("\n");
	    break;	  
  }
	  
  if(statistic) 
        stats.c.ipx++;

  return(RET_SUCCESS);
}
