#include "../include/aprecv-general.h"

int print_ospf_packet (unsigned char *packet, int opt_size) {
  static struct ospfv2_pkt_header *ospfv2_header;
  static struct ospfv3_pkt_header *ospfv3_header;
  static char *router_id, *area_id;
  static unsigned short auth_type;
  
  ospfv2_header = (struct ospfv2_pkt_header *)(packet+data_link_size+opt_size);
  
  apsr_output("- OSPF Header -\n  Version: %d\n", ospfv2_header->version);
  if(ospfv2_header->version == 2) {
	apsr_output("  Type: %d", ospfv2_header->type);

	switch(ospfv2_header->type) {
		case APSR_OPSF_TYPE_HELLO:
	          apsr_output(" (Hello)");
		  break;
		case APSR_OSPF_TYPE_DD:
		  apsr_output(" (Database Description)");
		  break;
		case APSR_OSPF_TYPE_LSR:
		  apsr_output(" (Link state request)");
		  break;
		case APSR_OSPF_TYPE_LSU:
		  apsr_output(" (Link state update)");
		  break;
		case APSR_OSPF_TYPE_LSA:
		  apsr_output(" (Link state acknowledgement)");
		  break;
		default:
		  apsr_output(" (unknown)");
		  break;
        }
		  
	apsr_output("\n  Length: %d", apsr_ntohs(ospfv2_header->length));
	
        router_id = strdup(inet_ntoa(ospfv2_header->router_id));
	area_id = strdup(inet_ntoa(ospfv2_header->area_id));

	apsr_output("\n  Router ID: %s", router_id);
/*	if(verbose) {
		resolve_ip(router_id, AF_INET);
        }
*/
        apsr_output("\n  Area ID: %s", area_id);
/*	if(verbose) {
	        resolve_ip(area_id, AF_INET);
	}
*/
	apsr_output("\n  Checksum: 0x%04x", apsr_ntohs(ospfv2_header->sum));

	auth_type = apsr_ntohs(ospfv2_header->autype);
	
	apsr_output("\n  Authentication Type: %d", auth_type);

	switch(auth_type) {
		case APSR_OSPF_AUTH_TYPE_NO:
		  apsr_output(" (None)");
		  break;
		case APSR_OSPF_AUTH_TYPE_SI:
		  apsr_output(" (Simple password authentication)");
		  break;
		case APSR_OSPF_AUTH_TYPE_CR:
		  apsr_output(" (Cryptographic authentication/MD5)");
		  break;
		default:
		  apsr_output(" (Reserved/Unknown)");
		  break;
	}

	apsr_output("\n");

	/* TODO Authentication */

	if(statistic)
		stats.c.ospfv2++;

        free(router_id);
        free(area_id);
  }
  else if(ospfv2_header->version == 3) {
	ospfv3_header = (struct ospfv3_pkt_header *)(packet+data_link_size+opt_size);  

        apsr_output("  Type: %d", ospfv3_header->type);
	
        switch(ospfv3_header->type) {
                case APSR_OPSF_TYPE_HELLO:
                  apsr_output(" (Hello)");
                  break;
                case APSR_OSPF_TYPE_DD:
                  apsr_output(" (Database Description)");
                  break;
                case APSR_OSPF_TYPE_LSR:
                  apsr_output(" (Link state request)");
                  break;
                case APSR_OSPF_TYPE_LSU:
                  apsr_output(" (Link state update)");
                  break;
                case APSR_OSPF_TYPE_LSA:
                  apsr_output(" (Link state acknowledgement)");
                  break;
                default:
                  apsr_output(" (unknown)");
                  break;
        }

        apsr_output("\n  Length: %d", apsr_ntohs(ospfv3_header->length));

	router_id = strdup(inet_ntoa(ospfv3_header->router_id));
        area_id = strdup(inet_ntoa(ospfv3_header->area_id));

        apsr_output("\n  Router ID: %s", router_id);
/*        if(verbose) {
              resolve_ip(router_id, AF_INET);
        }
*/
        apsr_output("\n  Area ID: %s", area_id);
/*        if(verbose) {
              resolve_ip(area_id, AF_INET);
	}	 
*/

        apsr_output("\n  Checksum: 0x%04x, Instance ID: %d\n", apsr_ntohs(ospfv3_header->sum), ospfv3_header->instance_id);

	if(statistic) 
		stats.c.ospfv3++;

        free(router_id);
	free(area_id);
  }
  
  
  return(RET_SUCCESS);
}
