#include "../include/aprecv-general.h"
#include "../include/aprecv-stream.h"

static struct tcp_stream *tcp_pkt_list=NULL;

int print_tcp_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size, int ip_version) {
  static struct tcp_pkt_header *tcp_header;
  static struct tcp_options *tcp_opt;
  static unsigned int sequence_num;
  static unsigned short src_port=0, dst_port=0, calulated_checksum;
  static unsigned int data_offset=0;
  unsigned int count_len=0, kind_pointer=0, i=0;
  struct tcp_stream *p;
  static unsigned char *options=NULL;

  tcp_header = (struct tcp_pkt_header *)(packet+data_link_size+opt_size);
  sequence_num = apsr_ntohl(tcp_header->seq_num);
  src_port = apsr_ntohs(tcp_header->src_port);
  dst_port = apsr_ntohs(tcp_header->dst_port);
  data_offset = tcp_header->data_offset * 4;

  apsr_output("- TCP Header -\n  Src Port: %d",(int) src_port);
  if(verbose) 
  	  apsr_output(" (%s)", apsr_resolve_port(src_port, "tcp"));

  apsr_output(", Dst Port: %d",(int) dst_port);
  if(verbose) 
  	  apsr_output(" (%s)", apsr_resolve_port(dst_port, "tcp"));
  
  apsr_output("\n  Seq.: %u", sequence_num);
  apsr_output("\n  Ack-Seq.: %u, Data-Offset: %d\n", apsr_ntohl(tcp_header->ack_num),(int) data_offset);

  apsr_output("  Flag(s): ");
  /* Check TCP Flags */
  if(tcp_header->flags & APSR_TCP_FIN) {
	  if(verbose) 
		  apsr_output("Fin ");
	  else 
		  apsr_output("F");
  }
  if(tcp_header->flags & APSR_TCP_SYN) { 
  	  if(verbose) 
		  apsr_output("Syn ");
	  else 
		  apsr_output("S");
  }
  if(tcp_header->flags & APSR_TCP_RST) {
	  if(verbose) 
		  apsr_output("Reset ");
	  else 
		  apsr_output("R");
  }
  if(tcp_header->flags & APSR_TCP_PSH) {
	  if(verbose) 
		  apsr_output("Push ");
	  else 
		  apsr_output("P");
  }
  if(tcp_header->flags & APSR_TCP_ACK) {
	  if(verbose) 
		  apsr_output("Ack ");
	  else 
		  apsr_output("A");
  }
  if(tcp_header->flags & APSR_TCP_URG) {
	  if(verbose) 
		  apsr_output("Urgent ");
	  else 
		  apsr_output("U");
  }
  if(tcp_header->flags & APSR_TCP_ECE) {
          if(verbose)
                  apsr_output("ECE ");
          else
                  apsr_output("E");
  }
  if(tcp_header->flags & APSR_TCP_CWR) {
          if(verbose)
                  apsr_output("CWR ");
          else
                  apsr_output("W");
  }
  apsr_output("\n  Win-Size: %d, Urg-Ptr: 0x%04x\n", apsr_ntohs(tcp_header->window), apsr_ntohs(tcp_header->urgent));
  
  apsr_output("  Checksum: 0x%04x", apsr_ntohs(tcp_header->sum));

  if(calc_checksums && ip_version == 4) {
	  calulated_checksum = apsr_tcpv4_checksum(tcp_header);
	  if(calulated_checksum == 0)
		  apsr_output(" (correct checksum)\n");
	  else {
		  apsr_output(" (incorrect checksum - should be 0x%04x)\n",
				  calulated_checksum);
		  if(statistic)
			  stats.cr.tcp++;
  	  }
  }
  else
	  apsr_output("\n");
    
  count_len = data_offset - 20;
  if(count_len > 0) {
	  apsr_output("  Options");
	  if(verbose) 
		  apsr_output("(%u bytes)", count_len);
	  apsr_output(": [");
	  options = (packet+data_link_size+opt_size+APSR_TCP_H);
	  for(i=0; i < count_len; i++) {
		  if(i == kind_pointer) {
  		    switch(options[i]) {
			  case APSR_TCP_OPTION_EOL:
			    if(verbose) 
				    apsr_output(" <End of List>");
			    else 
				    apsr_output(" <EOL>");
			    /* To avoid lood */
			    goto end_of_options;
			    /* Never reached */
			    break;
			  case APSR_TCP_OPTION_NOP:
			    if(verbose) 
				    apsr_output(" <No Operation>");
			    else 
				    apsr_output(" <NOP>");
			    kind_pointer+=1;
			    break;
			  case APSR_TCP_OPTION_MAXSEG:
			    tcp_opt=(struct tcp_options *)(packet+data_link_size+opt_size+APSR_TCP_H+i); 
			    if(verbose)
				    apsr_output(" <Maximum Segment Size ");
			    else
				    apsr_output(" <MSS ");
			    apsr_output("%u>", apsr_ntohs(tcp_opt->ext.mss.mss));
			    kind_pointer+=tcp_opt->ext.mss.length;
			    break;
			  case APSR_TCP_OPTION_WSCALE:
			    tcp_opt=(struct tcp_options *)(packet+data_link_size+opt_size+APSR_TCP_H+i);
			    if(verbose) 
				    apsr_output(" <Window Scale Factor ");
			    else 
				    apsr_output(" <WSopt ");
			    apsr_output("%u>", tcp_opt->ext.ws.ws);
			    kind_pointer+=tcp_opt->ext.ws.length;
			    break;
			  case APSR_TCP_OPTION_SACKOK:
			    if(verbose) 
				    apsr_output(" <Selective Ack ok>");
		    	    else 
				    apsr_output(" <SAck ok>");
			    kind_pointer+=2;
			    break;
			  case APSR_TCP_OPTION_SACK:
			    if(verbose)
				    apsr_output(" <Selective Ack>");
			    else 
				    apsr_output(" <SAck>");
			    kind_pointer+=4;	/* FIX: Variable */
			    break;
			  case APSR_TCP_OPTION_ECHO:
			    apsr_output(" <Echo>");
			    kind_pointer+=6;
			    break;
			  case APSR_TCP_OPTION_ECHOREPLY:
			    apsr_output(" <Echo Reply>");
			    kind_pointer+=6;
			    break;
			  case APSR_TCP_OPTION_TIMESTAMP:
			    tcp_opt=(struct tcp_options *)(packet+data_link_size+opt_size+APSR_TCP_H+i);
			    if(verbose)
				    apsr_output(" <Timestamp ");
			    else 
				    apsr_output(" <TSopt ");
			    apsr_output("TSval=%u TSecr=%u>",
		  	  	  apsr_ntohl(tcp_opt->ext.time.ts_value),
			          apsr_ntohl(tcp_opt->ext.time.ts_echo));
			    kind_pointer+=tcp_opt->ext.time.length;
			    break;
                          case APSR_TCP_OPTION_PARTIAL_PER:
                            apsr_output(" <Partial Order Con. Perm.>");
                            kind_pointer+=2;
                            break;
                          case APSR_TCP_OPTION_PARTIAL_PRO:
                            apsr_output(" <Partial Order Serv. Profile>");
                            kind_pointer+=3;
                            break;
                          case APSR_TCP_OPTION_CC:
			    tcp_opt=(struct tcp_options *)(packet+data_link_size+opt_size+APSR_TCP_H+i);
                            if(verbose)
				    apsr_output(" <Connection Count ");
			    else
			            apsr_output(" <CC ");
										    
                            apsr_output("%u>", apsr_ntohl(tcp_opt->ext.cc.cc));
                            kind_pointer+=tcp_opt->ext.cc.length;
                            break;
                          case APSR_TCP_OPTION_CC_NEW:
                            tcp_opt=(struct tcp_options *)(packet+data_link_size+opt_size+APSR_TCP_H+i);
                            if(verbose)
                                    apsr_output(" <Connection Count ");
                            else
	                            apsr_output(" <CC ");
 			    apsr_output("New %u>", apsr_ntohl(tcp_opt->ext.cc.cc));
                            kind_pointer+=tcp_opt->ext.cc.length;
                            break;
		          case APSR_TCP_OPTION_CC_ECHO:
                            tcp_opt=(struct tcp_options *)(packet+data_link_size+opt_size+APSR_TCP_H+i);
                            if(verbose)
	                                    apsr_output(" <Connection Count ");
                            else
	                                    apsr_output(" <CC ");
                            apsr_output("Echo %u>", apsr_ntohl(tcp_opt->ext.cc.cc));
                            kind_pointer+=tcp_opt->ext.cc.length;
                            break;
                          case APSR_TCP_OPTION_ALTER_CHK_R:
                            tcp_opt=(struct tcp_options *)(packet+data_link_size+opt_size+APSR_TCP_H+i);
                            apsr_output(" <Alternate Checksum Req. Type %d",(int) tcp_opt->ext.alter_chksum.type_sum);
                            if(verbose) {
                                   switch(tcp_opt->ext.alter_chksum.type_sum) {
                                     case APSR_TCP_OPT_ALTER_CHKSUM_TCP:
                                       apsr_output(" (TCP)");
                                       break;
                                     case APSR_TCP_OPT_ALTER_CHKSUM_8BIT:
                                       apsr_output(" (8-Bit)");
                                       break;
                                     case APSR_TCP_OPT_ALTER_CHKSUM_16BIT:
                                       apsr_output(" (16-Bit)");
                                       break;
                                     default:
                                       apsr_output(" (unknown)");
                                       break;
                                   }
                            }
                            apsr_output(">");                                       
                            kind_pointer+=3;
                            break;
                          case APSR_TCP_OPTION_ALTER_CHK_D:
                            apsr_output(" <Alternate Checksum Data>");
                            kind_pointer+=3;	/* FIX: Variable */
                            break;
			  case APSR_TCP_OPTION_TRAILER_CHK:
                            apsr_output(" <Trailer Checksum Option>");
                            kind_pointer+=3;
                            break;
			  case APSR_TCP_OPTION_MD5:
                            apsr_output(" <MD5 signature>");
                            kind_pointer+=18;
                            break;
			  default: 
			    break;
		    }
		  }		
	  }	  
end_of_options:
	  apsr_output(" ]\n");
  }
  
  if(tcp_header->flags & APSR_TCP_SYN && tcp_header->flags & APSR_TCP_RST &&
  tcp_header->flags & APSR_TCP_PSH && tcp_header->flags & APSR_TCP_FIN &&
  tcp_header->flags & APSR_TCP_ACK && tcp_header->flags & APSR_TCP_URG) {
	  apsr_output("  Error: protocol error in flags(all set)\n");
	  
	  if(statistic)
		  stats.err.tcp++;
  }
  else if(tcp_header->flags & APSR_TCP_SYN && 
		  tcp_header->flags & APSR_TCP_RST) {
	  apsr_output("  Error: protocol error in flags(SYN & RST)\n");

	  if(statistic)
		  stats.err.tcp++;
  }
  else if(!tcp_header->flags) {
	  apsr_output("  Error: no flags set\n");
	  
	  if(statistic)
		  stats.err.tcp++;
  }
 
  if(print_tcp_stream) {
    if(((tcp_header->flags & APSR_TCP_PSH && tcp_header->flags & APSR_TCP_ACK) || (tcp_header->flags == APSR_TCP_ACK) || (tcp_header->flags & APSR_TCP_PSH && tcp_header->flags & APSR_TCP_ACK && tcp_header->flags & APSR_TCP_FIN)) && (header->len-APSR_TCP_H-data_link_size-opt_size-count_len >= 0)) {
      if (!(p=xcalloc(1, sizeof(struct tcp_stream))) ) {
    	apsr_stream_output((FILE *) stderr, "Error while allocation memory for new module.\n");
      }
      
      /* Fill tcp_stream struct */
      
      /* IPv4 header */
      p->ipvhl	 	= temp_ipv4_stream_header->ipvhl;
      p->type_service	= temp_ipv4_stream_header->type_service;
      p->packet_len	= temp_ipv4_stream_header->packet_len;
      p->ident          = temp_ipv4_stream_header->ident;
      p->fragment	= temp_ipv4_stream_header->fragment;
      p->time_live	= temp_ipv4_stream_header->time_live;
      p->protocol	= temp_ipv4_stream_header->protocol;
      p->ip_sum		= temp_ipv4_stream_header->sum;
      p->src_ip    	= temp_ipv4_stream_header->src_ip;
      p->dst_ip   	= temp_ipv4_stream_header->dst_ip;

      /* TCP header */
      p->src_port  	= tcp_header->src_port;
      p->dst_port  	= tcp_header->dst_port;
      p->seq_num	= tcp_header->seq_num;
      p->ack_num	= tcp_header->ack_num;
      p->reserved	= tcp_header->reserved;
      p->data_offset 	= tcp_header->data_offset;
      p->flags		= tcp_header->flags;
      p->window		= tcp_header->window;
      p->tcp_sum	= tcp_header->sum;
      p->urgent		= tcp_header->urgent;
      
      /* Payload */
      p->payload_length = header->len-APSR_TCP_H-data_link_size-opt_size-count_len;
      strncpy(p->payload, packet+data_link_size+APSR_TCP_H+opt_size+count_len, header->len-APSR_TCP_H-data_link_size-opt_size-count_len);

      p->next      = tcp_pkt_list;
      tcp_pkt_list = p;
    }
  }
    
  if(((tcp_header->flags & APSR_TCP_PSH && tcp_header->flags & APSR_TCP_ACK) || (tcp_header->flags == APSR_TCP_ACK) || (tcp_header->flags & APSR_TCP_PSH && tcp_header->flags & APSR_TCP_ACK && tcp_header->flags & APSR_TCP_FIN)) && (header->len-APSR_TCP_H-data_link_size-opt_size-count_len >= 0)) {
 	if(src_port == APSR_TCP_PORT_BGP || dst_port == APSR_TCP_PORT_BGP) 
	  print_bgp_packet(packet, opt_size+APSR_TCP_H+opt_size+count_len);
	else if(src_port == APSR_TCP_PORT_MSDP || dst_port == APSR_TCP_PORT_MSDP)
	  print_msdp_packet(packet, opt_size+APSR_TCP_H+opt_size+count_len);
	else if(src_port == APSR_TCP_PORT_GSMP || dst_port == APSR_TCP_PORT_GSMP)
	  print_gsmp_packet(packet, opt_size+APSR_TCP_H+opt_size+count_len);
	#ifdef HAVE_APSRLIB_MODULE_SUPPORT
	else if(module_name || modules_in_path) {
  	  module_length = header->len-APSR_TCP_H-data_link_size-opt_size-count_len;
	  find_module(packet+data_link_size+APSR_TCP_H+opt_size+count_len, AF_INET, APSR_PROTO_TCP, APSR_PROTO_TCP, src_port, dst_port);
	}
	#endif	
  }

  if(print_payload_tcp_hex && !print_payload_tcp_text) {
	  print_raw_in_hex(packet+data_link_size+APSR_TCP_H+opt_size+count_len, header->len-APSR_TCP_H-data_link_size-opt_size);
  }
  else if(!print_payload_tcp_hex && print_payload_tcp_text) {
          print_raw_in_text(packet+data_link_size+APSR_TCP_H+opt_size+count_len, header->len-APSR_TCP_H-data_link_size-opt_size);	
  }

  if(statistic) 
          stats.c.tcp++;
  
  return(RET_SUCCESS);
}

void print_tcp_streams(void) {
	struct tcp_stream *l;

	l=tcp_pkt_list;

	apsr_output("\nCollected Payload: \"");

	rev_print_list(l);

	apsr_output("\"\n");
}

/* Recursive function */
void rev_print_list(struct tcp_stream *list) {
	unsigned short x=0;
	
	if(list != NULL) {
		rev_print_list(list->next);
		for(x = 0; x <= list->payload_length; x++) {
			if(isprint(list->payload[x]))
				apsr_output("%c", list->payload[x]);
			else 
				apsr_output(".");
			
		}
        }
}
