#include "../include/aprecv-general.h"
#include "aprecv-snoop.h"

void read_snoop(int max_packets) {
	size_t read_bytes;
	struct snoop_file_hdr *snoop_hdr;
	struct snoop_pkt_hdr *snoop_packet_header;
	struct aprecv_frame_header aprecv_header;
	unsigned int snoop_file_version, snoop_datalink_type, original_length,
	             included_length, record_length, pp_length,
		     timestamp_seconds, timestamp_microsec;
	unsigned char filehdr_buf[sizeof(struct snoop_file_hdr)];
	unsigned char packethdr_buf[sizeof(struct snoop_pkt_hdr)];
        unsigned int file_size, sec;
        int count = 0;
	unsigned char *packet;

        #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
        int *gzerror_num=NULL;
        #endif
		
	apsr_output("Read packet(s) from file: %s.\n", snoop_trace_file);

        #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
        if((file_handle = gzopen(snoop_trace_file, "r")) == NULL) {
	#else
	if((file_handle = fopen(snoop_trace_file, "r")) == NULL) {
	#endif
		apsr_output("Error: Can't open snoop file %s.\n",
				snoop_trace_file);
                exit(EXIT_FAILURE);
        }
	
        #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
        gzseek(file_handle, 0, SEEK_END);
        file_size = gztell(file_handle);

        /* Replacement for rewind */
        /* !!!FIXME!!! - gzseek seems not to set SEEK_SET */
        (void) gzseek(file_handle, 0L, SEEK_SET);
        #else
        fseek(file_handle, 0, SEEK_END);
        file_size = ftell(file_handle);

        /* Replacement for rewind */
        (void) fseek(file_handle, 0L, SEEK_SET);
        #endif

	#ifdef APRECV_DEBUG
  	apsr_output("Snoop file size is %d bytes.\n", file_size);
	#endif

        #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
        read_bytes = gzread(file_handle, filehdr_buf, sizeof(struct snoop_file_hdr));
        #else
        read_bytes = fread(filehdr_buf, 1, sizeof(struct snoop_file_hdr), file_handle);
        #endif
	
	if(read_bytes != sizeof(struct snoop_file_hdr)) 
		ERROR_EXIT("Error occured while Snoop file header reading.\n");
	
	snoop_hdr = (struct snoop_file_hdr *)filehdr_buf;
	
	#ifdef APRECV_DEBUG
	apsr_output("Check for snoop magic.\n");
	#endif
        if (strncmp(snoop_hdr->identification, snoop_identification, 8) == 0) {
         #ifdef APRECV_DEBUG		
         apsr_output("Found snoop magic.\n");
         apsr_output("Check for snoop version.\n");
         #endif
	 snoop_file_version = apsr_ntohl(snoop_hdr->version);
	 if(snoop_file_version == 2 || 
	    snoop_file_version == 3 ||
	    snoop_file_version == 4 ||
	    snoop_file_version == 5) {
          #ifdef APRECV_DEBUG
          apsr_output("Found snoop version %u.\n", snoop_file_version);
	  #endif
	  snoop_datalink_type = apsr_ntohl(snoop_hdr->datalink_type);
          while(1) {
            #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
   	    if((gztell(file_handle) == file_size)) {
            #else
            if((ftell(file_handle) == file_size)) {
            #endif
              apsr_output("\nRead snoop dump file completed.\n");
              goto close_it;
            }
    
            count++;

            #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
            read_bytes = gzread(file_handle, packethdr_buf, 
			    sizeof(struct snoop_pkt_hdr));
            #else
            read_bytes = fread(packethdr_buf, 1, 
			    sizeof(struct snoop_pkt_hdr), file_handle);
            #endif
            
	    if(read_bytes <= 0) {
              #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
              if(gzeof(file_handle))
              #else
              if(feof(file_handle)) 
              #endif
		      ERROR_EXIT("End of file reached, but not at the right position, maybe truncated.\n");

              #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
              if(gzerror(file_handle, gzerror_num))
                      apsr_output("Error: Read packet failed(%d).\n",
				      gzerror_num);
	      
	      #else
	      if(ferror(file_handle))
		      apsr_output("Error: Read packet failed.\n");
	      #endif
              goto close_it;
            }
            else {
	      snoop_packet_header = (struct snoop_pkt_hdr *)packethdr_buf;
	  
	      original_length = apsr_ntohl(snoop_packet_header->original_length);
	      included_length = apsr_ntohl(snoop_packet_header->included_length);
	      record_length = apsr_ntohl(snoop_packet_header->packet_record_length);

	      packet = (unsigned char *) xmalloc(included_length+1);
	      pp_length = record_length - sizeof(struct snoop_pkt_hdr);
              #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
              read_bytes = gzread(file_handle, packet,(size_t) pp_length);
              #else
              read_bytes = fread(packet, 1,(size_t) pp_length, file_handle);
              #endif
	
	      if(read_bytes <= 0) {
                #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
                if(gzeof(file_handle))
                #else
                if(feof(file_handle))
                #endif
		  ERROR_EXIT("End of file reached, but not at the right position, maybe truncated.\n");
                #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
                if(gzerror(file_handle, gzerror_num))
	        #else
	        if(ferror(file_handle))
		#endif
		  apsr_output("Error: Read packet failed.\n");
                goto close_it;
              }
              else {
	        stats.c.all++;

		timestamp_seconds = apsr_ntohl(snoop_packet_header->timestamp_seconds);
		timestamp_microsec = apsr_ntohl(snoop_packet_header->timestamp_microsec);
			
 		sec = timestamp_seconds % 86400;
		apsr_output("\n%02d:%02d:%02d - Read frame %lu(%u/%u bytes length)...\n", sec / 3600, (sec % 3600) / 60, sec % 60, stats.c.all, included_length, original_length);
		  	 
	        aprecv_header.caplen = included_length; 
                aprecv_header.len    = original_length;
		switch(snoop_datalink_type) {
		  case SNOOP_DT_8025:
		    data_link_size = 14;
		    print_token_packet(&aprecv_header, packet, 0);
	            break;
		  case SNOOP_DT_ETHERNET:
		  case SNOOP_DT_8023:
		    data_link_size = 14;
		    print_ethernet_packet(&aprecv_header, packet, 0);
		    break;
		  case SNOOP_DT_FDDI:
		    data_link_size = 21;
		    print_fddi_packet(&aprecv_header, packet, 0);
		    break;
		  default:
		    apsr_output("\n- Not support packet type -\n");
		    break;
	        }
		free(packet);
              }
              if(count == max_packets) {
		apsr_output("\nRead snoop dump file completed.\n");
		goto close_it;
              }
            }
	  }
         }
	 else {
		apsr_output("Unknown snoop version %u.\n", snoop_file_version);
         }
	}
        else {
		apsr_output("Unknown snoop file format(wrong magic).\n");
        }

close_it:

	/* Ugly stuff from src/error.c. But i don't know how to work with
	 * src/error.c - this gives a SIGSEGV ! */
	
        if(statistic)
		print_statistic();

	if(snoop_trace_file) {
                #ifdef HAVE_APSRLIB_LIBZ_SUPPORT
                if(gzclose(file_handle) < 0) {
	        #else
	        if(fclose(file_handle) < 0) {
		#endif
			apsr_stream_output((FILE *) stderr, "Error: Can't close tracefile %s.\n", snoop_trace_file);
            		#ifdef APRECV_DEBUG
		        apsr_stream_output((FILE *) stderr, "Error code: %s.\n", strerror(errno));
	                #endif
	    	}
        }

	#ifdef HAVE_APSRLIB_MODULE_SUPPORT
	close_modules();
	#endif

	if(server_addr)
		close_server_connection();

	exit(EXIT_SUCCESS);
}
