#include "../include/apsrstack-general.h"

void process_arp_packet (void *packet, int opt_size) {
  struct apsrstack_arp_pkt_header *arp_header;
  unsigned short hardware_type, protocol_type, opcode;
  unsigned char *temp_buf;

  arp_header = (struct apsrstack_arp_pkt_header *)(packet+data_link_size+opt_size);

  hardware_type = apsr_ntohs(arp_header->arp_hat);
  protocol_type = apsr_ntohs(arp_header->arp_pat);
  opcode        = apsr_ntohs(arp_header->arp_oc);

  if(hardware_type != APSR_ARPHRD_ETHER) {
	/* Currently we only support Ethernet */
	stats.arp.drops++; /* Drops++ */
        return; 
  }

  if(protocol_type != APSR_ETHERTYPE_IP) {
	/* Currently we only support IPv4 */
	stats.arp.drops++; /* Drops */
        return; 
  }

  if(arp_header->arp_hal != APSR_ETHER_ADDR_LEN) {
	/* Currently we only support Ethernet */
        stats.arp.drops++; /* Drops++ */
        return;
  }	

  if(arp_header->arp_pal != 4) {
        /* Currently we only support IPv4 */
        stats.arp.drops++; /* Drops++ */
        return;
  }
  
  if(opcode == APSR_ARPOP_REQUEST) {
	  /* Is ARP Request for my IP Address ? */
	  if(arp_header->target_pr_addr[0] == APSRSTACK_IP_1 &&
	     arp_header->target_pr_addr[1] == APSRSTACK_IP_2 &&
	     arp_header->target_pr_addr[2] == APSRSTACK_IP_3 &&
	     arp_header->target_pr_addr[3] == APSRSTACK_IP_4) {
		  
	    send_arp(arp_header->sender_hw_addr, apsrstack_src_mac, ETHERTYPE_ARP, 
		      APSR_ARPHRD_ETHER, APSR_ETHERTYPE_IP, 6, 4, 
		      APSR_ARPOP_REPLY, apsrstack_src_mac, apsrstack_src_ip, 
		      arp_header->sender_hw_addr, arp_header->sender_pr_addr);

	    stats.arp.processed++; /* Counter++ */
	    return;
	  }
          else {
 	    stats.arp.drops++; /* Drops++ */
  	    return;
          }	
  }  
}

void send_arp(unsigned char *ethernet_dst, unsigned char *ethernet_src,
              unsigned short ethernet_type, unsigned short arp_hdr_ether,
	      unsigned short arp_type, unsigned char arp_hal, 
	      unsigned char arp_pal, unsigned short arp_opcode,
	      unsigned char *arp_sha, unsigned char *arp_spa,
	      unsigned char *arp_tha, unsigned char *arp_tpa) {
	
	struct apsr_link_layer *apsr_ll;
	unsigned char *buf;
  
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

        apsr_init_psize(APSR_ETH + APSR_ARP, &buf);

        apsr_ethernet(ethernet_dst, ethernet_src, ethernet_type, NULL, 0, buf);

        apsr_arp(arp_hdr_ether, arp_type, arp_hal, arp_pal, arp_opcode, arp_sha, arp_spa, arp_tha, arp_tpa, NULL, 0, buf + APSR_ETH);

        apsr_write_link_layer(apsr_ll, device, buf, APSR_ETH + APSR_ARP, 1);

	apsr_close_link_layer(apsr_ll);
        free(buf);
}	
		
