#include "../include/apsrlib-config.h"

/* Create an ARP packet. */
int apsr_arp (unsigned short hat, unsigned short pat, unsigned char hal,
	      unsigned char pal, unsigned short oc, unsigned char *sha,
	      unsigned char *spa, unsigned char *tha,
	      unsigned char *tpa, const unsigned char *payload,
	      int payload_size, unsigned char *buf)
{
	
	struct arp_pkt_header arp_hdr;
	if(!buf) {
	  #ifdef APSRLIB_DEBUG
	  ERROR("apsr_arp: !buf");
	  #endif
	  return(RET_FAILURE);
	}

	arp_hdr.arp_hat = apsr_htons(hat);	/* hardware address type   */
	arp_hdr.arp_pat = apsr_htons(pat);	/* protocol address type   */
	arp_hdr.arp_hal = hal;			/* hardware address length */
	arp_hdr.arp_pal = pal;			/* protocol address length */
	arp_hdr.arp_oc  = apsr_htons(oc);	/* opcode command	   */

	/* Check the payload size and copy it to the buffer. */
	if(payload && payload_size) {
	  memcpy(buf + APSR_ARP, payload, payload_size);
	}

	/*  Copy the ARP header to the buffer. */
	memcpy(buf, &arp_hdr, sizeof(arp_hdr));

	
	/* Sender hardware address */
	memcpy(buf+payload_size+sizeof(arp_hdr), sha, hal);
	
	/* Sender protocol address */
	memcpy(buf+payload_size+sizeof(arp_hdr)+hal, spa, pal);
	
	/* Target hardware address */
	memcpy(buf+payload_size+sizeof(arp_hdr)+hal+pal, tha, hal);

	/* Target protocol address */
	memcpy(buf+payload_size+sizeof(arp_hdr)+hal+pal+hal, tpa, pal);

	return(RET_SUCCESS);
}
