#include "../include/apsrlib-config.h"

/* NOTE:							*/
/* RAW sockets: Checksum is always computed by the kernel.	*/
/* Link layer : Checksum must be explicitly computed.		*/
/*								*/
/* TODO: IPv6 checksums.					*/
int apsr_checksum (unsigned char *buf, unsigned char protocol, unsigned short len) {
	struct ipv4_pkt_header *ipv4_hdr;

	int checksum, ipv4_hl;

	/* Pseudo checksum */
	checksum = 0;

	/* Get the IPv4 header */
	ipv4_hdr = (struct ipv4_pkt_header *) buf;
	ipv4_hl = APSR_IPV4_HEADERLENGTH(ipv4_hdr)*4;

	/* Switch the transport layer protocol. */
	switch (protocol) {
		case APSR_PROTO_TCP: { 
		  struct tcp_pkt_header *tcp_hdr;
		  tcp_hdr = (struct tcp_pkt_header *) (buf + ipv4_hl);

		  /* Solaris checksum bug. */
		  #ifdef SOLARIS_CHECKSUM_BUG
		  tcp_hdr->sum = tcp_hdr->data_offset << 2;
		  return(RET_SUCCESS);
		  #endif
		  
		  tcp_hdr->sum = 0;
		  checksum = pseudo_in_cksum((unsigned short *) &ipv4_hdr->src_ip, 8);
		  checksum += ntohs(APSR_PROTO_TCP + len);
		  checksum += pseudo_in_cksum((unsigned short *) tcp_hdr, len);
		  tcp_hdr->sum = APSR_CKSUM(checksum);
		  }  
		  break;
		case APSR_PROTO_UDP: {
		  struct udp_pkt_header *udp_hdr;
		  udp_hdr = (struct udp_pkt_header *) (buf + ipv4_hl);
		  
		  udp_hdr->sum = 0x0000;
		  checksum = pseudo_in_cksum((unsigned short *) &ipv4_hdr->src_ip, 8);
		  checksum += ntohs(APSR_PROTO_UDP + len);
		  checksum += pseudo_in_cksum((unsigned short *) udp_hdr, len);
		  udp_hdr->sum = APSR_CKSUM(checksum);
  	          }
		  break;
		case APSR_PROTO_EGP: {
		  struct egp_pkt_header *egp_hdr;
		  egp_hdr = (struct egp_pkt_header *) (buf + ipv4_hl);
		  egp_hdr->sum = 0x0000;
		  checksum = in_cksum((unsigned short *)egp_hdr,
		  			len, 0);
		  egp_hdr->sum = checksum;
		  }
		  break;
		case APSR_PROTO_IRTP: {
		  struct irtp_pkt_header *irtp_hdr;
		  irtp_hdr = (struct irtp_pkt_header *) (buf + ipv4_hl);
		  irtp_hdr->sum = 0x0000;
		  checksum = in_cksum((unsigned short *)irtp_hdr,
		  			len, 0);
		  irtp_hdr->sum = checksum;
		  }
		  break;
		case APSR_PROTO_IGRP: {
		  struct igrp_pkt_header *igrp_hdr;
		  igrp_hdr = (struct igrp_pkt_header *) (buf + ipv4_hl);
		  igrp_hdr->sum = 0x0000;
		  checksum = in_cksum((unsigned short *)igrp_hdr,
		  			len, 0);
		  igrp_hdr->sum = checksum;
		  }
		  break;
                case APSR_PROTO_EIGRP: {
                  struct eigrp_pkt_header *eigrp_hdr;
                  eigrp_hdr = (struct eigrp_pkt_header *) (buf + ipv4_hl);
	          eigrp_hdr->sum = 0x0000;
	          checksum = in_cksum((unsigned short *)eigrp_hdr,
                                        len, 0);
                  eigrp_hdr->sum = checksum;
                  }
		  break;
		case APSR_PROTO_IGMP: {
		  struct igmpv2_pkt_header *igmpv2_hdr;
		  igmpv2_hdr = (struct igmpv2_pkt_header *) (buf + ipv4_hl);
		  igmpv2_hdr->sum = 0x0000;
		  checksum = in_cksum((unsigned short *)igmpv2_hdr,
		  			len, 0);
		  igmpv2_hdr->sum = checksum;
		  }
		  break;
		case APSR_PROTO_NARP: {
		  struct narp_pkt_header *narp_hdr;
		  narp_hdr = (struct narp_pkt_header *) (buf + ipv4_hl);
		  narp_hdr->sum = 0x0000;
		  checksum = in_cksum((unsigned short *)narp_hdr,
		  			len, 0);
		  narp_hdr->sum = checksum;
		  }
		  break;
		case APSR_PROTO_IFMP: {
		  struct ifmp_pkt_header *ifmp_hdr;
		  ifmp_hdr = (struct ifmp_pkt_header *) (buf + ipv4_hl);
		  ifmp_hdr->sum = 0x0000;
		  checksum = in_cksum((unsigned short *)ifmp_hdr,
		  			len, 0);
		  ifmp_hdr->sum = checksum;
		  }
		  break;
		case APSR_PROTO_PIM: {
		  struct pim_pkt_header *pim_hdr;
		  pim_hdr = (struct pim_pkt_header *) (buf + ipv4_hl);
		  pim_hdr->sum = 0x0000;
		  checksum = in_cksum((unsigned short *)pim_hdr,
		  			len, 0);
		  pim_hdr->sum = checksum;
		  }
		  break;
                case APSR_PROTO_ICMP: {
                  struct icmpv4_pkt_header *icmpv4_hdr;
                  icmpv4_hdr = (struct icmpv4_pkt_header *) (buf + ipv4_hl);
                  icmpv4_hdr->sum = 0x0000;
                  checksum = in_cksum((unsigned short *)icmpv4_hdr, len, 0);
                  icmpv4_hdr->sum = checksum;
                  }
                  break;
		default: 
		  #ifdef APSRLIB_DEBUG
		  ERROR("apsr_checksum: Unknown transport layer protocol!");
		  #endif

		  return(RET_FAILURE);
	}

	return(RET_SUCCESS);
}

/* Alternating checksum routine for UDP over IPv4 */
int apsr_udpv4_checksum(struct udp_pkt_header *udp_header) {
	unsigned short *sp;

	v4_ps_hdr.ps_hdr.padding 	= 0;
        v4_ps_hdr.ps_hdr.protocol 	= APSR_PROTO_UDP;

        sp = &v4_ps_hdr.pa[0];
	
	/* ntohs(v4_ps_hdr.ps_hdr.length) is not the best way */
        return in_cksum((unsigned short *)udp_header, 
			ntohs(v4_ps_hdr.ps_hdr.length),
			sp[0]+sp[1]+sp[2]+sp[3]+sp[4]+sp[5]);
}

/* Alternating checksum routine for TCP over IPv4 */
int apsr_tcpv4_checksum(struct tcp_pkt_header *tcp_header) {
	unsigned short *sp;

        v4_ps_hdr.ps_hdr.padding       = 0;
        v4_ps_hdr.ps_hdr.protocol      = APSR_PROTO_TCP;

        sp = &v4_ps_hdr.pa[0];

	/* ntohs(v4_ps_hdr.ps_hdr.length) is not the best way */
        return in_cksum((unsigned short *)tcp_header,
		       	ntohs(v4_ps_hdr.ps_hdr.length),
		       	sp[0]+sp[1]+sp[2]+sp[3]+sp[4]+sp[5]);
}

