#include "../include/apsrlib-config.h"

/* Create an ICMPv4 packet. */
int apsr_icmpv4 (unsigned char type, unsigned char code,
		 unsigned short sum, const unsigned char *payload, 
		 int payload_size, unsigned char *buf) {
	static struct icmpv4_pkt_header icmpv4_hdr;

	if(!buf) {
	  #ifdef APSRLIB_DEBUG
	  ERROR("apsr_icmpv4: !buf");
	  #endif
	  return(RET_FAILURE);
	}

	icmpv4_hdr.type		= type;
	icmpv4_hdr.code		= code;
	icmpv4_hdr.sum		= apsr_htons(sum);

	/* Check the payload size and copy it to the buffer. */
	if(payload && payload_size) {
	  memcpy(buf + APSR_ICMPv4_H, payload, payload_size);
	}

	memcpy((unsigned char *)buf, (unsigned char *)&icmpv4_hdr, APSR_ICMPv4_H);

	return(RET_SUCCESS);
}

/* Create an ICMPv4 Echo packet. */
int apsr_icmpv4_echo (unsigned char type, unsigned char code,
		      unsigned short sum, unsigned short identifier,
		      unsigned short seq_num, const unsigned char *payload, 
		      int payload_size, unsigned char *buf) {
	static struct icmpv4_pkt_header icmpv4_hdr;

	if(!buf) {
	  #ifdef APSRLIB_DEBUG
	  ERROR("apsr_icmpv4_echo: !buf");
	  #endif
	  return(RET_FAILURE);
	}

	icmpv4_hdr.type		= type;
	icmpv4_hdr.code		= code;
	icmpv4_hdr.sum		= apsr_htons(sum);
	icmpv4_hdr.ext.echo.identifier 	= apsr_htons(identifier);
	icmpv4_hdr.ext.echo.seq_num 	= apsr_htons(seq_num);

	/* Check the payload size and copy it to the buffer. */
	if(payload && payload_size) {
	  memcpy(buf + APSR_ICMPv4_H + 4, payload, payload_size);
	}

	memcpy((unsigned char *)buf, (unsigned char *)&icmpv4_hdr, APSR_ICMPv4_H + 4);

	return(RET_SUCCESS);
}

/* Create an ICMPv4 Timestamp packet. */
int apsr_icmpv4_timestamp (unsigned char type, unsigned char code,
		      unsigned short sum, unsigned short identifier,
		      unsigned short seq_num, unsigned int originate,
		      unsigned int receive, unsigned int transmit,	      
		      const unsigned char *payload, 
		      int payload_size, unsigned char *buf) {
	static struct icmpv4_pkt_header icmpv4_hdr;

	if(!buf) {
	  #ifdef APSRLIB_DEBUG
	  ERROR("apsr_icmpv4_timestamp: !buf");
	  #endif
	  return(RET_FAILURE);
	}

	icmpv4_hdr.type		= type;
	icmpv4_hdr.code		= code;
	icmpv4_hdr.sum		= apsr_htons(sum);
	icmpv4_hdr.ext.timestamp.identifier 	= apsr_htons(identifier);
	icmpv4_hdr.ext.timestamp.seq_num 	= apsr_htons(seq_num);
	icmpv4_hdr.ext.timestamp.originate	= apsr_htonl(originate);
	icmpv4_hdr.ext.timestamp.receive	= apsr_htonl(receive);
	icmpv4_hdr.ext.timestamp.transmit	= apsr_htonl(transmit);	

	/* Check the payload size and copy it to the buffer. */
	if(payload && payload_size) {
	  memcpy(buf + APSR_ICMPv4_H + 16, payload, payload_size);
	}

	memcpy((unsigned char *)buf, (unsigned char *)&icmpv4_hdr, APSR_ICMPv4_H + 16);

	return(RET_SUCCESS);
}

