#include "../include/apsrlib-config.h"

/* Create a RIPv1/v2 packet. */
int apsr_rip (unsigned int command, unsigned int version,
	      unsigned short routing_domain, unsigned short address_family,
	      unsigned short route_tag, unsigned long ip_address,
	      unsigned long subnet_mask, unsigned long next_hop_ip,
	      unsigned int metric, const unsigned char *payload,
	      int payload_size, unsigned char *buf)
{
	static struct rip_pkt_header rip_hdr;

	if(!buf) {
	  #ifdef APSRLIB_DEBUG
          ERROR("apsr_rip: !buf");
	  #endif
	  return(RET_FAILURE);
	}

	rip_hdr.command			= command;
	rip_hdr.version			= version;
	rip_hdr.routing_domain		= apsr_htons(routing_domain);
	rip_hdr.address_family		= apsr_htons(address_family);
	rip_hdr.route_tag		= apsr_htons(route_tag);
	rip_hdr.ip_address.s_addr	= ip_address;
	rip_hdr.subnet_mask.s_addr	= subnet_mask;
	rip_hdr.next_hop_ip.s_addr	= next_hop_ip;
	rip_hdr.metric			= apsr_htonl(metric);
	
	
	/* Check the payload size and copy it to the buffer. */
	if(payload && payload_size) {
	  memcpy(buf + APSR_RIP, payload, payload_size);
	}

	memcpy((unsigned char *)buf, (unsigned char *)&rip_hdr,
	       sizeof(rip_hdr));

	return(RET_SUCCESS);
}

