#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.03"
#define MODULE_NAME		"icp"
#define MODULE_DESCRIPTION	"Example ICP (Internet Cache Protocol) protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	3130	
#define MODULE_IDENT_ID2	3130

#define MODULE_FILTER		"udp and port 3130"

static const char *mod_icp_opcode[]= {
	" (Invalid)",
	" (Query)",
	" (Hit)",
	" (Miss)",
	" (Error)",
	" (Send)",
        " (SendA)",
        " (Data Beginning)",
        " (Data)",
        " (Data End)",
	" (SEcho)",
	" (DEcho)",
        " (Notify)",
	" (Invalidate)",
	" (Purge)",
	" (Wiretap)",
	" (Unused)",
	" (Unused)",
	" (Missing Pointer)",
	" (Advertise)",
	" (Unadvertise)",
	" (Missing no fetch)",
        " (Denied)"
	" (Hit object)",	
};

#define APSR_ICP_OPCODE_INVALID		0	/* rfc 2186 */
#define APSR_ICP_OPCODE_QUERY		1	/* rfc 2186 */
#define APSR_ICP_OPCODE_HIT		2	/* rfc 2186 */
#define APSR_ICP_OPCODE_MISS		3	/* rfc 2186 */
#define APSR_ICP_OPCODE_ERR		4	/* rfc 2186 */
#define APSR_ICP_OPCODE_SEND		5
#define APSR_ICP_OPCODE_SENDA		6
#define APSR_ICP_OPCODE_DATA_BEGINNING	7
#define APSR_ICP_OPCODE_DATA		8
#define APSR_ICP_OPCODE_DATA_END	9
#define APSR_ICP_OPCODE_SECHO		10	/* rfc 2186 */
#define APSR_ICP_OPCODE_DECHO		11	/* rfc 2186 */
#define APSR_ICP_OPCODE_NOTIFY		12	
#define APSR_ICP_OPCODE_INVALIDATE	13	
#define APSR_ICP_OPCODE_PURGE		14	
#define APSR_ICP_OPCODE_WIRETAP		15
#define APSR_ICP_OPCODE_MISS_POINTER	18
#define APSR_ICP_OPCODE_ADVERTISE	19
#define APSR_ICP_OPCODE_UNADVERTISE	20
#define APSR_ICP_OPCODE_MISS_NOFETCH	21	/* rfc 2186 */
#define APSR_ICP_OPCODE_DENIED		22	/* rfc 2186 */
#define APSR_ICP_OPCODE_HIT_OBJ		23	/* rfc 2186 */

struct mod_icp_header {
	unsigned char opcode;
	unsigned char version;
	unsigned short length;
	unsigned int req_num;
	unsigned int options;
	unsigned int option_data;
	struct in_addr sender_host_addr;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
  	module_info.family      = MODULE_IDENT_FAMILY;
    	module_info.proto1      = MODULE_IDENT_PN1;
      	module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = 0;
	module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_icp_header *icp_header;
  char *sender_ip;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    icp_header = (struct mod_icp_header *)buf;
	    sender_ip = strdup(inet_ntoa(icp_header->sender_host_addr));
	    apsr_output("- ICP Header -\n");
	    apsr_output("  Opcode: %u", icp_header->opcode);
	    if(verbose) 
		    apsr_output("%s", mod_icp_opcode[icp_header->opcode]);
            apsr_output("  Version: %u\n", icp_header->version);
	    apsr_output("  Message length: %u, Request number: %u\n",
			    apsr_ntohs(icp_header->length), apsr_ntohl(icp_header->req_num));
	    apsr_output("  Options: 0x%04x, Options Data: 0x%04x\n", 
			    apsr_ntohl(icp_header->options), apsr_ntohl(icp_header->option_data));
	    apsr_output("  Sender host Address: %s\n", sender_ip);
	    free(sender_ip);
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
