#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"imap"
#define MODULE_DESCRIPTION	"Example Interactive Mail Access Protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	143	/* v2 */
#define MODULE_IDENT_ID2	143	/* v2 */
#define MODULE_IDENT_ID3	220	/* v3/v4 */
#define MODULE_IDENT_ID4        220     /* v3/v4 */

/* TODO: v3/v4, currently only v2(rfc 1176) conform */

#define MODULE_FILTER		"tcp and (port 143 or 220)"

#define APSR_MOD_IMAP_TAG_COMMAND	6	/* Length of "a0001 " */
#define APSR_MOD_IMAP_TAG_RESPONSE	2	/* Length of "* " */

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID2 == id2 || MODULE_IDENT_ID4 == id2)) {
	    apsr_output("- IMAP(Client->Server) -\n");
            if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "APPEND", 6) == 0)
		    apsr_output("  Command: APPEND\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "AUTHENTICATE", 12) == 0) 
	            apsr_output("  Command: AUTHENTICATE\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "CAPABILITY", 10) == 0)
		    apsr_output("  Command: CAPABILITY\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "CHECK", 5) == 0)
	            apsr_output("  Command: CHECK\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "CLOSE", 5) == 0)
	            apsr_output("  Command: CLOSE\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "COPY", 4) == 0)
	            apsr_output("  Command: COPY\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "CREATE", 6) == 0)
	            apsr_output("  Command: CREATE\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "DELETE", 6) == 0)
	            apsr_output("  Command: DELETE\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "DELETEACL", 9) == 0)
	            apsr_output("  Command: DELETEACL\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "EXAMINE", 7) == 0)
	            apsr_output("  Command: EXAMINE\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "EXPUNGE", 7) == 0)
                    apsr_output("  Command: EXPUNGE\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "FETCH", 5) == 0)
                    apsr_output("  Command: FETCH\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "GETACL", 6) == 0)
                    apsr_output("  Command: GETACL\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "GETQUOTA", 8) == 0)
                    apsr_output("  Command: GETQUOTA\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "GETQUOTAROOT", 12) == 0)
                    apsr_output("  Command: GETQUOTAROOT\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "LIST", 4) == 0)
                    apsr_output("  Command: LIST\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "LISTRIGHTS", 10) == 0)
                    apsr_output("  Command: LISTRIGHTS\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "LOGIN", 5) == 0)
                    apsr_output("  Command: LOGIN\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "LOGOUT", 6) == 0)
                    apsr_output("  Command: LOGOUT\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "LSUB", 4) == 0)
                    apsr_output("  Command: LSUB\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "MYRIGHTS", 8) == 0)
                    apsr_output("  Command: MYRIGHTS\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "NOOP", 4) == 0)
                    apsr_output("  Command: NOOP\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "RENAME", 6) == 0)
                    apsr_output("  Command: RENAME\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "SEARCH", 6) == 0)
                    apsr_output("  Command: SEARCH\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "SELECT", 6) == 0)
                    apsr_output("  Command: SELECT\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "SETACL", 6) == 0)
                    apsr_output("  Command: SETACL\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "SETQUOTA", 8) == 0)
                    apsr_output("  Command: SETQUOTA\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "STATUS", 6) == 0)
                    apsr_output("  Command: STATUS\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "STORE", 5) == 0)
                    apsr_output("  Command: STORE\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "SUBSCRIBE", 9) == 0)
                    apsr_output("  Command: SUBSCRIBE\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "UID", 3) == 0)
                    apsr_output("  Command: UID\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "UNSUBSCRIBE", 11) == 0)
                    apsr_output("  Command: UNSUBSCRIBE\n");
  }
  else if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID3 == id1)) {
            apsr_output("- IMAP(Server->Client) -\n");
            if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "OK", 2) == 0)
		    apsr_output("  Response: OK\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "NO", 2) == 0) 
	            apsr_output("  Response: NO\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_COMMAND, "BAD", 3) == 0)
		    apsr_output("  Response: BAD\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_RESPONSE, "FLAGS", 5) == 0)
	            apsr_output("  Response: FLAGS\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_RESPONSE, "SEARCH", 6) == 0)
	            apsr_output("  Response: SEARCH\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_RESPONSE, "BBOARD", 6) == 0)
	            apsr_output("  Response: BBOARD\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_RESPONSE, "MAILBOX", 7) == 0)
                    apsr_output("  Response: MAILBOX\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_RESPONSE, "BYE", 3) == 0)
                    apsr_output("  Response: BYE\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_RESPONSE, "OK", 2) == 0)
                    apsr_output("  Response: OK\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_RESPONSE, "NO", 2) == 0)
		    apsr_output("  Response: NO\n");
            else if(strncmp((char *)buf+APSR_MOD_IMAP_TAG_RESPONSE, "BAD", 3) == 0)
                    apsr_output("  Response: BAD\n");
  }
	  
  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
