#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.03"
#define MODULE_NAME		"radius"
#define MODULE_DESCRIPTION	"Example RADIUS (Remote Authentication Dial-in User Service) protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	1646	
#define MODULE_IDENT_ID2	1812
#define MODULE_IDENT_ID3	1813

#define MODULE_FILTER		"udp and (port 1646 or port 1812 or port 1813)"

/* TODO: Move all codes below into a switch case */

#define APSR_RADIUS_CODE_ACCESS_REQ		1	
#define APSR_RADIUS_CODE_ACCESS_ACC		2	
#define APSR_RADIUS_CODE_ACCESS_REJ		3	
#define APSR_RADIUS_CODE_ACCOUNTING_REQ		4	
#define APSR_RADIUS_CODE_ACCOUNTING_RES		5	
#define APSR_RADIUS_CODE_ACCESS_PASSWD_REQ	7
#define APSR_RADIUS_CODE_ACCESS_PASSWD_ACK	8
#define APSR_RADIUS_CODE_ACCESS_PASSWD_REJ	9
#define APSR_RADIUS_CODE_ACCESS_CHA		11	
#define APSR_RADIUS_CODE_STATUS_SER		12	/* experimental */
#define APSR_RADIUS_CODE_STATUS_CLI		13	/* experimental */
#define APSR_RADIUS_CODE_VENDOR_SPEC_CODE	26
#define APSR_RADIUS_CODE_ASCEND_ACC_NEXT_CO	29
#define APSR_RADIUS_CODE_ASCEND_ACC_NEW_PIN	30
#define APSR_RADIUS_CODE_ASCEND_PASSWD_EXPI	32
#define APSR_RADIUS_CODE_ASCEND_ACC_EVE_REQ	33
#define APSR_RADIUS_CODE_ASCEND_ACC_EVE_REP	34
#define APSR_RADIUS_CODE_DISCONNECT_REQ		40
#define APSR_RADIUS_CODE_DISCONNECT_REQ_ACK	41
#define APSR_RADIUS_CODE_DISCONNECT_REQ_NAK	42
#define APSR_RADIUS_CODE_CHG_FILTER_REQ		43
#define APSR_RADIUS_CODE_CHG_FILTER_REQ_ACK	44
#define APSR_RADIUS_CODE_CHG_FILTER_REQ_NAK	45
#define APSR_RADIUS_CODE_RESERVED		255

struct mod_radius_header {
	unsigned char code;
	unsigned char identifier;
	unsigned short length;
	unsigned char authenticator[16];
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
  	module_info.family      = MODULE_IDENT_FAMILY;
   	module_info.proto1      = 0;
      	module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = MODULE_IDENT_ID3;
	module_info.id4         = 0;
	      
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_radius_header *radius_header;
  unsigned short i;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2 || (MODULE_IDENT_ID3 == id1 || MODULE_IDENT_ID3 == id2))) {
	    radius_header = (struct mod_radius_header *)buf;
	    apsr_output("- RADIUS Header -\n");
	    apsr_output("  Code: %u", radius_header->code);
	    if(verbose) {
		    switch(radius_header->code) {
			    case APSR_RADIUS_CODE_ACCESS_REQ:
				    apsr_output(" (Access-request)");
				    break;
			    case APSR_RADIUS_CODE_ACCESS_ACC:
				    apsr_output(" (Access-accept)");
				    break;
			    case APSR_RADIUS_CODE_ACCESS_REJ:
				    apsr_output(" (Access-reject)");
				    break;
                            case APSR_RADIUS_CODE_ACCOUNTING_REQ:
			            apsr_output(" (Accounting-request)");
			            break;
			    case APSR_RADIUS_CODE_ACCOUNTING_RES:
			            apsr_output(" (Accounting-response)");
			            break;
                            case APSR_RADIUS_CODE_ACCESS_CHA:
                                    apsr_output(" (Access-challenge)");
                                    break;
                            case APSR_RADIUS_CODE_STATUS_SER:
                                    apsr_output(" (Status-server)");
                                    break;
                            case APSR_RADIUS_CODE_STATUS_CLI:
                                    apsr_output(" (Status-client)");
                                    break;
			    case APSR_RADIUS_CODE_RESERVED:
				    apsr_output(" (Reserved)");
				    break;
			    default:
				    apsr_output(" (Unknown)");
				    break;
		    }
	    }
            apsr_output("\n  Identifier: %u\n", radius_header->identifier);
	    apsr_output("  Length: %u\n", apsr_ntohs(radius_header->length));
	    apsr_output("  Authenticator: \"");
            for(i = 0; i <= 15; i++) {
		    if(isprint(radius_header->authenticator[i]))
			    apsr_output("%c", radius_header->authenticator[i]);
                    else
                            apsr_output(".");
            }
	    apsr_output("\"\n");
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
